#!/bin/sh

:<<'########################################################################'

  Usage: make_chrony.sh
    
This script downloads and constructs the chrony extension. See

------------------------------------------------------------------------

Change-log: 
  2011-10-09 First version.

------------------------------------------------------------------------

This script was written in 2011 by vitex from forum.tinycorelinux.net.

To the extent possible under law, the author(s) have dedicated all
copyright and related and neighboring rights to this software to the
public domain worldwide. This software is distributed without any
warranty.

See http://creativecommons.org/publicdomain/zero/1.0/ for a copy 
of the CC0 Public Domain Dedication under which this work is 
distributed.

########################################################################

# Specify the name of the extension.
EXTENSION=chrony

# Specify the base portion of the extension.
EXTENSION0=chrony

#-----------------------------------------------------------------------

# Install the extensions needed to build this extension.
tce-load -wi compiletc.tcz groff.tcz

#-----------------------------------------------------------------------

# Specify the temporary directory that will contain the build components.
TMP=/tmp/tcz

# Specify the directory in which the squashfs files will be assembled.
SQUASHFS=$TMP/squashfs 

# Specify the directory in which the submission package will be assembled.
PACKAGE=$TMP/package

#-----------------------------------------------------------------------

# Specify the location containing the software distribution.
VERSION=1.26
DOWNLOAD=http://download.tuxfamily.org/chrony/chrony-$VERSION.tar.gz

#-----------------------------------------------------------------------

# Download if necessary.
tar tzf $TMP/$(basename $DOWNLOAD) 1>/dev/null 2>&1 ||
  wget -O- $DOWNLOAD >$TMP/$(basename $DOWNLOAD)

#-----------------------------------------------------------------------

# Save the download for GPL compliance.
cp $TMP/$(basename $DOWNLOAD) $PACKAGE

#-----------------------------------------------------------------------

# Compute the name of the directory into which the software will be unpacked.
DIRECTORY=$TMP/$EXTENSION0-$VERSION

#-----------------------------------------------------------------------

# Unpack the software.
rm -fR $DIRECTORY
tar xzf $TMP/$(basename $DOWNLOAD) -C $TMP

#-----------------------------------------------------------------------

export CFLAGS="-march=i486 -mtune=i686 -Os -pipe"
export CXXFLAGS="-march=i486 -mtune=i686 -Os -pipe"
export LDFLAGS="-Wl,-O1"

#-----------------------------------------------------------------------

# Build the software.
{ cd $DIRECTORY ; ./configure ; make ; 
  touch $TMP/TIMESTAMP ; sudo make install ; }

#-----------------------------------------------------------------------

# Copy the installed files (those newer than $TMP/TIMESTAMP) from 
# /usr/local to $SQUASHFS/usr/local.
( cd / 
  find usr/local -not -type d -newer $TMP/TIMESTAMP |
    grep -v /man/ | grep -v /doc/ |
    cpio -pvd $SQUASHFS
)

#-----------------------------------------------------------------------

# Make a default configuration file.
mkdir -p $SQUASHFS/usr/local/etc/$EXTENSION
cat >$SQUASHFS/usr/local/etc/$EXTENSION/$EXTENSION.conf.sample <<EOF
# Use the specified servers to estimate the time when chronyd starts.
# If the offset is more than 30 seconds, step the clock to the correct
# time; otherwise, slew the clock.
initstepslew 30 0.pool.ntp.org 1.pool.ntp.org 2.pool.ntp.org 3.pool.ntp.org

# Use the following servers.
server 0.pool.ntp.org     minpoll 5
server 1.pool.ntp.org     minpoll 5
server 2.pool.ntp.org     minpoll 5
server 3.pool.ntp.org     minpoll 5

# Specify the key file and command key.
keyfile /usr/local/etc/chrony/chrony.keys
commandkey 01
EOF

#-----------------------------------------------------------------------

# Create /usr/local/etc/chrony/chrony.keys.sample.
cat >$SQUASHFS/usr/local/etc/$EXTENSION/$EXTENSION.keys.sample <<EOF 
01 mc
EOF
chmod 600 $SQUASHFS/usr/local/etc/$EXTENSION/EXTENSION.keys.sample

#-----------------------------------------------------------------------

# Create a script to start and stop chronyd.
mkdir -p $SQUASHFS/usr/local/etc/init.d
cat >$SQUASHFS/usr/local/etc/init.d/$EXTENSION <<'EOF'
#!/bin/sh

# chrony start script

[ $(id -u) = 0 ] || { echo "must be root" ; exit 1; }

start(){
  /usr/local/sbin/chronyd -f /usr/local/etc/chrony/chrony.conf
}

stop(){
  kill $(pidof chronyd)
}

restart(){
  pidof chronyd >/dev/null && stop
  start
}

case $1 in
  start)   start ;;
  stop)    stop ;;
  restart) restart ;;
  *)       echo "Usage $0 {start|stop|restart}"; exit 1
esac
EOF
chmod +x $SQUASHFS/usr/local/etc/init.d/$EXTENSION

#-----------------------------------------------------------------------

# Create an initialization script that creates default configuration 
# files if the user has not supplied them and then start a background
# to activate chrony once DNS is available.
mkdir -p $SQUASHFS/usr/local/tce.installed
cat >$SQUASHFS/usr/local/tce.installed/$EXTENSION <<EOF
#!/bin/sh

# Create /usr/local/etc/chrony/chrony.conf if the user has not supplied it.
[ ! -r /usr/local/etc/chrony/chrony.conf ] &&
  cp /usr/local/etc/chrony/chrony.conf.sample /usr/local/etc/chrony/chrony.conf

# Create /usr/local/etc/chrony/chrony.keys if the user has not supplied it.
[ ! -r /usr/local/etc/chrony/chrony.keys ] &&
  cp /usr/local/etc/chrony/chrony.keys.sample /usr/local/etc/chrony/chrony.keys

# Start a background job to wait until dns starts working before
# starting chronyd and then to issue a "burst" command to accelerate
# time synchronization.
( while ! nslookup pool.ntp.org >/dev/null 2>&1 ; do sleep 1 ; done ; sleep 2
  /usr/local/sbin/chronyd -f /usr/local/etc/chrony/chrony.conf 
  sleep 10 ; echo -e 'password mc\nburst 5/10\nquit\n' | chronyc
)&
EOF
sudo chmod +x $SQUASHFS/usr/local/tce.installed/$EXTENSION

#-----------------------------------------------------------------------

# Create $EXTENSION.tcz.dep.
( echo readline.tcz ; echo libcap.tcz ) >$PACKAGE/$EXTENSION.tcz.dep

#-----------------------------------------------------------------------

# Create $EXTENSION.tcz.info.
cat >$PACKAGE/$EXTENSION.tcz.info <<EOF
Title:          $EXTENSION.tcz
Description:    chrony - programs for keeping computer clocks accurate
Version:        $VERSION
Author:         Richard Curnow (original version)
Original-site:  http://chrony.tuxfamily.org/
Copying-policy: GNU General Public License Version 2
Size:           SiZeK
Extension_by:   vitex
Comments:       From http://chrony.tuxfamily.org/:

                  Chrony is a pair of programs which are used to
                  maintain the accuracy of the system clock on a
                  computer. The two programs are called chronyd and
                  chronyc.

                  Chronyd is a daemon which runs in background on the
                  system. It obtains measurements via the network of
                  the system clock's offset relative to time servers
                  on other systems and adjusts the system time
                  accordingly. For isolated systems, the user can
                  periodically enter the correct time by hand (using
                  Chronyc). In either case, Chronyd determines the
                  rate at which the computer gains or loses time, and
                  compensates for this. Chronyd implements the NTP
                  protocol and can act as either a client or a server.

                  Chronyc provides a user interface to Chronyd for
                  monitoring its performance and configuring various
                  settings. It can do so while running on the same
                  computer as the Chronyd instance it is controlling
                  or a different computer.

            
                -----------------------------------------------------

                This extension was built from 

                  $DOWNLOAD

                The chrony manual is

                  http://chrony.tuxfamily.org/manual.html

                The chrony configuration file for this extension is 

                  /usr/local/etc/chrony/chrony.conf

                It should work without modification for any system
                that has good network connectivity when the system
                starts.  Execute

                  sudo /usr/local/etc/init.d/chrony restart

                to restart chrony if the network goes down or if
                chrony does not seem to be synchronizing the clock.
                The default configuration does not save any system
                state on disk.  Chrony will generally synchronize the
                system clock within a few minutes.

                The default configuration synchronizes with four
                servers from the pool provided by pool.ntp.org; see
                http://www.pool.ntp.org for suggestions about
                selecting more appropriate servers.

                The default configuration tests the accuracy of the
                system clock whenever chrony starts execution.  If the
                clock is in error by more than 30 seconds, the clock
                is set immediately to the correct time. Otherwise, the
                clock is slewed (adjusted) incrementally in such a
                manner that time never jumps backward.

                Execute 

                  echo -e 'tracking\n sources -v' | chronyc 

                to get an overview of the state of chronyd.

                Critical chronyc commands are protected by a password
                that is stored in

                  /usr/local/etc/chrony/chrony.keys

                The default password is "mc". For example, execute

                  echo -e 'password mc\n burst 5/10' | chronyc 

                to cause chronyd to attempt to get 5 good responses 
                quickly for each ntp server but to send no more than
                10 requests to each server.  This command can speed
                up synchronization.

                -----------------------------------------------------

Change-log:     ----------
          	2010/01/03 First version
         	2010/04/03 Upgraded to 1.24 and added chrony-doc.tcz
            	2011/12/14 Upgraded to 1.26
Current:	2011/12/17 Added libcap.tcz dependency
EOF

#-----------------------------------------------------------------------
